#include "cam_preprocess.h"

CamPreprocess::CamPreprocess(uint8_t id)
{
    this->id = id;
}

CamPreprocess::~CamPreprocess()
{
    int ret;
    for (uint8_t i = 0; i < this->inputPicInfo.size(); i++)
    {
        ret = hi_mpi_dvpp_free(this->inputPicInfo[i].picture_address);
        if (!ret)
        {
            this->logger.LogInfo() << "release memory of input image " << i;
        }
        else
        {
            this->logger.LogError() << "failed to release memory of input image " << i << " with value: " << ret;
        }

        ret = hi_mpi_dvpp_free(this->outputPicInfo[i].picture_address);
        if (!ret)
        {
            this->logger.LogInfo() << "release memory of output image " << i;
        }
        else
        {
            this->logger.LogError() << "failed to release memory of output image " << i << " with value: " << ret;
        }

        ret = hi_mpi_dvpp_free(this->cropResizeRegion[i].dest_pic_info.picture_address);
        if (!ret)
        {
            this->logger.LogInfo() << "release memory of cropResize image " << i;
        }
        else
        {
            this->logger.LogError() << "failed to release memory of cropResize image " << i << " with value: " << ret;
        }

        ret = hi_mpi_vpc_destroy_chn(this->channelIDs[i]);
        if (!ret)
        {
            this->logger.LogInfo() << "destroy channel " << this->channelIDs[i];
        }
        else
        {
            this->logger.LogError() << "failed to destroy channel " << this->channelIDs[i] << " with value: " << ret;
        }
    }

    ret = hi_mpi_sys_exit();
    if (!ret)
    {
        this->logger.LogInfo() << "Media data processing system de-initialization";
    }
    else
    {
        this->logger.LogError() << "Media data processing system de-initialization failed";
    }

    ret = aclrtDestroyContext(this->g_context);
    if (!ret)
    {
        this->logger.LogInfo() << "release Context";
    }
    else
    {
        this->logger.LogError() << "release Context failed";
    }

    ret = aclrtResetDevice(0);
    if (!ret)
    {
        this->logger.LogInfo() << "release Device";
    }
    else
    {
        this->logger.LogError() << "release Device failed";
    }

    ret = aclFinalize();
    if (!ret)
    {
        this->logger.LogInfo() << "AscendCL de-initializes";
    }
    else
    {
        this->logger.LogError() << "AscendCL de-initializes failed";
    }
}

void CamPreprocess::setDefaultRemapParam(uint8_t channel_count)
{
    this->mappingParameters.resize(channel_count);
    this->remappingTableInformation.resize(channel_count);
    this->warpTransformParam.resize(channel_count);
    hi_u32 size;
    for (uint8_t i = 0; i < channel_count; i++)
    {
        this->mappingParameters[i].src_pic_width = this->inputImageWidth;
        this->mappingParameters[i].src_pic_height = this->inputImageHeight;
        this->mappingParameters[i].dest_pic_width = this->inputImageWidth;
        this->mappingParameters[i].dest_pic_height = this->inputImageHeight;
        this->mappingParameters[i].map_size = this->mappingParameters[i].dest_pic_width * this->mappingParameters[i].dest_pic_height * sizeof(hi_float);

        this->remappingTableInformation[i].src_pic_width = this->inputImageWidth;
        this->remappingTableInformation[i].src_pic_height = this->inputImageHeight;
        this->remappingTableInformation[i].dest_pic_width = this->inputImageWidth;
        this->remappingTableInformation[i].dest_pic_height = this->inputImageHeight;
        uint8_t failed = hi_mpi_vpc_get_lut_mem_size(this->remappingTableInformation[i].dest_pic_width, this->remappingTableInformation[i].dest_pic_height, &size);
        if (failed)
        {
            this->logger.LogError() << "[ERROR] hi_mpi_vpc_get_lut_mem_size failed with value: " << failed;
        }
        this->remappingTableInformation[i].lut_size = size;

        this->warpTransformParam[i].src.picture_width = this->inputImageWidth;
        this->warpTransformParam[i].src.picture_height = this->inputImageHeight;
        this->warpTransformParam[i].src.picture_format = HI_PIXEL_FORMAT_YUV_SEMIPLANAR_420;
        this->warpTransformParam[i].src.picture_width_stride = this->inputImageWidth;
        this->warpTransformParam[i].src.picture_height_stride = this->inputImageHeight;
        this->warpTransformParam[i].src.picture_buffer_size = this->warpTransformParam[i].src.picture_width_stride * this->warpTransformParam[i].src.picture_height_stride * 3 / 2;
        this->warpTransformParam[i].dst.picture_width = this->inputImageWidth;
        this->warpTransformParam[i].dst.picture_height = this->inputImageHeight;
        this->warpTransformParam[i].dst.picture_format = HI_PIXEL_FORMAT_YUV_SEMIPLANAR_420;
        this->warpTransformParam[i].dst.picture_width_stride = this->inputImageWidth;
        this->warpTransformParam[i].dst.picture_height_stride = this->inputImageHeight;
        this->warpTransformParam[i].dst.picture_buffer_size = this->warpTransformParam[i].dst.picture_width_stride * this->warpTransformParam[i].dst.picture_height_stride * 3 / 2;
        this->warpTransformParam[i].transform_config.interpolation = 0;
        this->warpTransformParam[i].transform_config.border_type = HI_BORDER_REPLICATE;

        this->workSpaceParam[i].func = REMAP;
        this->workSpaceParam[i].max_source_pic_width = this->inputImageWidth;
        this->workSpaceParam[i].max_source_pic_height = this->inputImageHeight;
        this->workSpaceParam[i].max_dest_pic_width = this->inputImageWidth;
        this->workSpaceParam[i].max_dest_pic_height = this->inputImageHeight;
    }
}

void CamPreprocess::setDefaultInputPicInfo(uint8_t channel_count)
{
    this->inputPicInfo.resize(channel_count);
    for (uint8_t i = 0; i < channel_count; i++)
    {
        this->inputPicInfo[i].picture_width = this->inputImageWidth;
        this->inputPicInfo[i].picture_height = this->inputImageHeight;
        this->inputPicInfo[i].picture_format = HI_PIXEL_FORMAT_YUV_SEMIPLANAR_420;
        this->inputPicInfo[i].picture_width_stride = this->inputImageWidth;
        this->inputPicInfo[i].picture_height_stride = this->inputImageHeight;
        this->inputPicInfo[i].picture_buffer_size = this->inputPicInfo[i].picture_width_stride * this->inputPicInfo[i].picture_height_stride * 3 / 2;
    }
}

void CamPreprocess::setDefaultCropReizeRegion(uint8_t channel_count)
{
    this->cropResizeRegion.resize(channel_count);
    for (uint8_t i = 0; i < channel_count; i++)
    {
        this->cropResizeRegion[i].dest_pic_info.picture_width = this->cropWidth;
        this->cropResizeRegion[i].dest_pic_info.picture_height = this->cropHeight;
        this->cropResizeRegion[i].dest_pic_info.picture_format = HI_PIXEL_FORMAT_YUV_SEMIPLANAR_420;
        this->cropResizeRegion[i].dest_pic_info.picture_width_stride = this->cropWidth;
        this->cropResizeRegion[i].dest_pic_info.picture_height_stride = this->cropHeight;
        this->cropResizeRegion[i].dest_pic_info.picture_buffer_size = this->cropResizeRegion[i].dest_pic_info.picture_width_stride * this->cropResizeRegion[i].dest_pic_info.picture_height_stride * 3 / 2;
        this->cropResizeRegion[i].crop_region.top_offset = this->cropTopOffset;
        this->cropResizeRegion[i].crop_region.left_offset = this->cropLeftOffset;
        this->cropResizeRegion[i].crop_region.crop_width = this->inputImageWidth - this->cropResizeRegion[i].crop_region.left_offset * 2;
        this->cropResizeRegion[i].crop_region.crop_height = this->inputImageHeight - this->cropResizeRegion[i].crop_region.top_offset * 2;
        this->cropResizeRegion[i].resize_info.resize_width = this->cropWidth;
        this->cropResizeRegion[i].resize_info.resize_height = this->cropHeight;
        this->cropResizeRegion[i].resize_info.interpolation = 1;
    }
}

void CamPreprocess::setDefaultOutputPicInfo(uint8_t channel_count)
{
    this->outputPicInfo.resize(channel_count);
    for (uint8_t i = 0; i < channel_count; i++)
    {
        this->outputPicInfo[i].picture_width = this->cropResizeRegion[i].dest_pic_info.picture_width;
        this->outputPicInfo[i].picture_height = this->cropResizeRegion[i].dest_pic_info.picture_height;
        this->outputPicInfo[i].picture_format = HI_PIXEL_FORMAT_BGR_888;
        this->outputPicInfo[i].picture_width_stride = this->cropResizeRegion[i].dest_pic_info.picture_width * 3;
        this->outputPicInfo[i].picture_height_stride = this->cropResizeRegion[i].dest_pic_info.picture_height;
        this->outputPicInfo[i].picture_buffer_size = this->outputPicInfo[i].picture_width_stride * this->outputPicInfo[i].picture_height_stride;
    }
}

void CamPreprocess::init(uint8_t channel_count, unsigned int inputImageWidth, unsigned int inputImageHeight, unsigned int cropWidth, unsigned int cropHeight, unsigned int cropTopOffset, unsigned int cropLeftOffset, bool initEncoder, unsigned int encoderGopNumber)
{
    uint8_t failed;
    this->inputImageWidth = inputImageWidth;
    this->inputImageHeight = inputImageHeight;
    this->cropWidth = cropWidth;
    this->cropHeight = cropHeight;
    this->cropTopOffset = cropTopOffset;
    this->cropLeftOffset = cropLeftOffset;
    this->encoderGopNumber = encoderGopNumber;
    failed = this->initACL();
    this->initMPI(channel_count);
    if (!failed)
    {
        this->logger.LogInfo() << "initialize pipeline done";
    }
    else
    {
        this->logger.LogError() << "initialize pipeline failed with result = 0x" << failed;
    }
    if (initEncoder)
    {
        this->createEncoderChannel(this->inputImageWidth, this->inputImageHeight, 5, channel_count);
    }
}

uint8_t CamPreprocess::initACL()
{
    aclError aclRet;
    aclRet = aclrtGetRunMode(&this->runMode);
    if (!aclRet)
    {
        this->logger.LogInfo() << "ACL get run mode successfully";
    }
    else
    {
        this->logger.LogError() << "ACL didn't get run mode successfully, ret = 0x" << aclRet;
    }
    aclRet = aclInit(nullptr);
    if (!aclRet)
    {
        this->logger.LogInfo() << "ACL Initialized successfully";
    }
    else
    {
        this->logger.LogError() << "ACL didn't initialized successfully, ret = 0x" << aclRet;
    }
    aclRet = aclrtSetDevice(0);
    if (!aclRet)
    {
        this->logger.LogInfo() << "ACL set device successfully";
    }
    else
    {
        this->logger.LogError() << "ACL didn't set device successfully, ret = 0x" << aclRet;
    }
    aclRet = aclrtCreateContext(&this->g_context, 0);
    if (!aclRet)
    {
        this->logger.LogInfo() << "ACL context successfully";
    }
    else
    {
        this->logger.LogError() << "ACL didn't context successfully, ret = 0x" << aclRet;
    }
    return 0;
}

int32_t CamPreprocess::getResizedCroppedImage(uint8_t cameraId)
{
    int32_t failed = 0;
    uint32_t taskID = 0;
    failed = hi_mpi_vpc_crop_resize(this->channelIDs[cameraId], &this->inputPicInfo[cameraId], &this->cropResizeRegion[cameraId], 1, &taskID, -1);
    if (!failed)
    {
        failed = hi_mpi_vpc_get_process_result(this->channelIDs[cameraId], taskID, -1);
        if (failed)
        {
            this->logger.LogError() << "failed to get cropped and resized image with result 0x" << failed;
        }
    }
    else
    {
        this->logger.LogError() << "failed to crop and resize with result 0x" << failed;
    }
    return failed;
}

int32_t CamPreprocess::getConvertedImage(uint8_t cameraId)
{
    int32_t failed = 0;
    uint32_t taskID = 0;
    failed = hi_mpi_vpc_convert_color(this->channelIDs[cameraId], &this->cropResizeRegion[cameraId].dest_pic_info, &this->outputPicInfo[cameraId], &taskID, -1);
    if (!failed)
    {
        failed = hi_mpi_vpc_get_process_result(this->channelIDs[cameraId], taskID, -1);
        if (failed)
        {
            this->logger.LogError() << "failed to get converted image with result 0x" << failed;
        }
    }
    else
    {
        this->logger.LogError() << "failed to convert with result 0x" << failed;
    }
    return failed;
}

int32_t CamPreprocess::getUndistortedImage(uint8_t cameraId)
{
    int32_t failed = 0;
    uint32_t taskID = 0;
    failed = hi_mpi_vpc_set_chn_workspace(this->channelIDs[cameraId], this->workSpaceParam, 1);
    if (failed)
    {
        this->logger.LogError() << "applied for temporary cache failed with result = 0x" << failed;
    }
    else
    {
        this->logger.LogInfo() << "channel " << this->channelIDs[cameraId] << " applied for temporary cache successfully";
    }
    failed = hi_mpi_vpc_get_remap_lut(&this->mappingParameters[cameraId], &this->remappingTableInformation[cameraId]);
    if (failed)
    {
        this->logger.LogError() << "failed to get remap lut with result 0x" << failed;
    }
    failed = hi_mpi_vpc_lut_remap(this->channelIDs[cameraId], &this->warpTransformParam[cameraId], &this->remappingTableInformation[cameraId], &taskID, -1);
    if (!failed)
    {
        failed = hi_mpi_vpc_get_process_result(this->channelIDs[cameraId], taskID, -1);
        if (failed)
        {
            this->logger.LogError() << "failed to get undistorted image with result 0x" << failed;
        }
    }
    else
    {
        this->logger.LogError() << "failed to undistort with result 0x" << failed;
    }
    return failed;
}

int32_t CamPreprocess::createVPCChannel()
{
    int32_t failed = 0;
    for (uint8_t i = 0; i < this->channelIDs.size(); i++)
    {
        failed = hi_mpi_vpc_sys_create_chn(&this->channelIDs[i], &this->channelAttr[i]);
        if (failed)
        {
            return failed;
        }
        else
        {
            this->logger.LogInfo() << "channel " << this->channelIDs[i] << " created successfully";
        }
    }
    return failed;
}

uint8_t CamPreprocess::allocateByPicInfo(uint8_t id, std::vector<hi_vpc_pic_info> &picInfo)
{
    int32_t failed = 0;
    failed = hi_mpi_dvpp_malloc(0, &picInfo[id].picture_address, picInfo[id].picture_buffer_size);
    if (failed)
    {
        this->logger.LogError() << "failed to allocate dvpp memory for channel with index" << id << " with result 0x" << failed;
    }
    return failed;
}

uint8_t CamPreprocess::allocateByCropResizeRegion(uint8_t id, std::vector<hi_vpc_crop_resize_region> &cropResizeRegion)
{
    int32_t failed = 0;
    failed = hi_mpi_dvpp_malloc(0, &cropResizeRegion[id].dest_pic_info.picture_address, cropResizeRegion[id].dest_pic_info.picture_buffer_size);
    if (failed)
    {
        this->logger.LogError() << "failed to allocate dvpp memory for crop resize region in channel with index" << id << " with result 0x" << failed;
    }
    return failed;
}

uint32_t CamPreprocess::allocateMemory(uint32_t id)
{
    uint8_t failed;
    failed = this->allocateByPicInfo(id, this->inputPicInfo);
    if (!failed)
    {
        this->logger.LogInfo() << "input memory allocated successfully for channel: " << id;
        failed = this->allocateByPicInfo(id, this->outputPicInfo);
        if (!failed)
        {
            this->logger.LogInfo() << "output memory allocated successfully for channel: " << id;
            failed = this->allocateByCropResizeRegion(id, this->cropResizeRegion);
            if (!failed)
            {
                this->logger.LogInfo() << "crop and resize memory allocated successfully for channel: " << id;
            }
            else
            {
                this->logger.LogError() << "crop and resize memory allocated failed for channel: " << id << " with result: " << failed;
            }
        }
        else
        {
            this->logger.LogError() << "output memory allocated failed for channel: " << id << " with result: " << failed;
        }
    }
    else
    {
        this->logger.LogError() << "input memory allocated failed for channel: " << id << " with result: " << failed;
    }
    return failed;
}

uint8_t CamPreprocess::initMPI(uint8_t channel_count)
{
    int32_t failed = hi_mpi_sys_init();

    if (!failed)
    {
        this->channelIDs.resize(channel_count);
        this->channelAttr.resize(channel_count);
        for (uint8_t i = 0; i < channel_count; i++)
        {
            this->channelAttr[i].pic_width = 0;
            this->channelAttr[i].pic_height = 0;
        }
        this->setDefaultInputPicInfo(channel_count);
        this->setDefaultCropReizeRegion(channel_count);
        this->setDefaultOutputPicInfo(channel_count);
        this->setDefaultRemapParam(channel_count);
        failed = this->createVPCChannel();
        if (!failed)
        {
            this->logger.LogInfo() << "VPC channels created successfully" << this->loggerName;
        }
        else
        {
            this->logger.LogFatal() << "VPC channels created failed with result: " << failed << " in: " << this->loggerName;
        }
    }
    else
    {
        this->logger.LogFatal() << "mpi sys init failed with result: " << failed;
    }
    return failed;
}

int32_t CamPreprocess::createEncoderChannel(int width, int height, int bufferSize, uint8_t channel_count)
{
    int32_t ret;

    this->epollFd.resize(channel_count);
    this->event.resize(channel_count);
    this->fd.resize(channel_count);

    // set venc parameter
    hi_venc_mod_param mod_param{};
    mod_param.mod_type = HI_VENC_MOD_H265;
    hi_mpi_venc_get_mod_param(&mod_param);
    mod_param.h265_mod_param.one_stream_buf = 1;
    hi_mpi_venc_set_mod_param(&mod_param);

    for (uint8_t i = 0; i < channel_count; i++)
    {
        // Create channel
        hi_venc_chn chn = this->id;
        hi_venc_chn_attr attr{};
        attr.venc_attr.type = HI_PT_H265;
        attr.venc_attr.profile = 0;
        attr.venc_attr.max_pic_width = width;
        attr.venc_attr.max_pic_height = height;
        attr.venc_attr.pic_width = width;
        attr.venc_attr.pic_height = height;
        attr.venc_attr.buf_size = bufferSize * 1024 * 1024;
        attr.venc_attr.is_by_frame = HI_TRUE;
        attr.rc_attr.rc_mode = HI_VENC_RC_MODE_H265_VBR;
        attr.rc_attr.h265_vbr.gop = this->encoderGopNumber;
        attr.rc_attr.h265_vbr.stats_time = 1;
        attr.rc_attr.h265_vbr.src_frame_rate = 30;
        attr.rc_attr.h265_vbr.dst_frame_rate = 30;
        attr.rc_attr.h265_vbr.max_bit_rate = 4000;
        attr.gop_attr.gop_mode = HI_VENC_GOP_MODE_NORMAL_P;
        attr.gop_attr.normal_p.ip_qp_delta = 3;
        ret = hi_mpi_venc_create_chn(chn, &attr);
        this->logger.LogInfo() << "encoder create channel returned 0x" << ret;

        hi_venc_start_param recv_param{};
        recv_param.recv_pic_num = -1;
        hi_mpi_venc_start_chn(chn, &recv_param);

        // 1.Create EPOLL instance
        this->epollFd[i] = 0;
        this->fd[i] = hi_mpi_venc_get_fd(chn);
        ret = hi_mpi_sys_create_epoll(10, &this->epollFd[i]);
        this->event[i].events = HI_DVPP_EPOLL_IN;
        this->event[i].data = (void *)(unsigned long)(this->fd[i]);
        ret = hi_mpi_sys_ctl_epoll(this->epollFd[i], HI_DVPP_EPOLL_CTL_ADD, this->fd[i], &this->event[i]);
    }

    return ret;
}

uint8_t CamPreprocess::h265Encoder(std::shared_ptr<hi_venc_stream> *out_enc, hi_venc_chn _chn)
{
    int32_t ret = 0;

    // If the operating mode is ACL_HOST, you need to apply for Host memory, read the input data into the Host memory, and then transfer the Host data to the Device through the aclrtMemcpy interface. After the data transmission is completed, the Host memory needs to be released in time; otherwise, the input data will be directly Read into Device memory
    if (this->runMode == ACL_HOST)
    {
        // not supported in sdk 007
        this->logger.LogError() << "ACL_HOST mode is not supported";
    }
    else
    {

        // Send input data and start encoding
        hi_video_frame_info frame{};
        frame.mod_id = HI_ID_VENC;
        frame.v_frame.width = this->inputImageWidth;
        frame.v_frame.height = this->inputImageHeight;
        frame.v_frame.field = HI_VIDEO_FIELD_FRAME;
        frame.v_frame.pixel_format = HI_PIXEL_FORMAT_YUV_SEMIPLANAR_420;
        frame.v_frame.video_format = HI_VIDEO_FORMAT_LINEAR;
        frame.v_frame.compress_mode = HI_COMPRESS_MODE_NONE;
        frame.v_frame.dynamic_range = HI_DYNAMIC_RANGE_SDR8;
        frame.v_frame.color_gamut = HI_COLOR_GAMUT_BT709;
        frame.v_frame.width_stride[0] = this->inputImageWidth;
        frame.v_frame.width_stride[1] = this->inputImageWidth;
        frame.v_frame.width_stride[2] = this->inputImageWidth;
        frame.v_frame.virt_addr[0] = this->inputPicInfo[_chn].picture_address;
        frame.v_frame.virt_addr[1] = (hi_void *)((uintptr_t)frame.v_frame.virt_addr[0] + (this->inputImageWidth * this->inputImageHeight));
        frame.v_frame.virt_addr[2] = (hi_void *)((uintptr_t)frame.v_frame.virt_addr[2] + 1);
        this->frame_ref += 2;
        frame.v_frame.frame_flag = 0;
        frame.v_frame.time_ref = this->frame_ref;
        frame.v_frame.pts = this->cam_data.auxInfo.ispTimeStamp.timesec * 1000000000L + this->cam_data.auxInfo.ispTimeStamp.timensec;

        ret = hi_mpi_venc_send_frame(this->id, &frame, -1);
        if (ret == 0)
        {
            // Get the encoding result
            int32_t eventCount = 0;
            // Before the encoding is completed, it will time out and block here. Once it is completed, it will continue to execute.
            ret = hi_mpi_sys_wait_epoll(this->epollFd[_chn], &this->event[_chn], 3, 1000, &eventCount);
            // 2 Get encoding results
            hi_venc_chn_status stat;
            ret = hi_mpi_venc_query_status(this->id, &stat);
            auto temp = this->id;
            if (ret == 0)
            {
                hi_venc_pack *pack_ptr = new hi_venc_pack[stat.cur_packs];
                auto enc_stream_ptr = std::shared_ptr<hi_venc_stream>(
                    new hi_venc_stream, [temp, pack = pack_ptr](auto *ptr)
                    {
                            if (0 != hi_mpi_venc_release_stream(temp, ptr)) {
                                std::cout << "hi_mpi_venc_release_stream failed" << std::endl;
                            }
                            delete[] pack;
                            delete ptr; });

                enc_stream_ptr->pack_cnt = stat.cur_packs;
                enc_stream_ptr->pack = pack_ptr;
                hi_mpi_venc_get_stream(temp, enc_stream_ptr.get(), -1);
                *out_enc = enc_stream_ptr;
            }
        }
    }
    return (uint8_t)ret;
}

int32_t CamPreprocess::runEncoderPipeline(uint8_t cameraId, std::shared_ptr<hi_venc_stream> *out_enc)
{
    int32_t failed = this->h265Encoder(out_enc, cameraId);
    if (!failed)
    {
        this->logger.LogDebug() << "image encoded to H.265 successfully";
    }
    else
    {
        this->logger.LogError() << "image H.265 encoding error with result: 0x" << failed;
    }
    return failed;
}

int32_t CamPreprocess::runPerceptionPipeline(uint8_t cameraId)
{
    int32_t failed;
    failed = this->getResizedCroppedImage(cameraId);
    if (!failed)
    {
        this->logger.LogDebug() << "image cropped and resized successfully";
        failed = this->getConvertedImage(cameraId);
        if (!failed)
        {
            this->logger.LogDebug() << "image converted to BGR successfully";
        }
        else
        {
            this->logger.LogError() << "image converting to BGR error with result: 0x" << failed;
            return failed;
        }
    }
    else
    {
        this->logger.LogError() << "image cropping and resizing error with result: 0x" << failed;
        return failed;
    }
    return failed;
}

int32_t CamPreprocess::runPipeline(uint8_t cameraId, std::shared_ptr<hi_venc_stream> *out_enc)
{
    int32_t failed;
    failed = this->getResizedCroppedImage(cameraId);
    if (!failed)
    {
        this->logger.LogDebug() << "image cropped and resized successfully";
        failed = this->getConvertedImage(cameraId);
        if (!failed)
        {
            this->logger.LogDebug() << "image converted to BGR successfully";
            failed = this->h265Encoder(out_enc, cameraId);
            if (!failed)
            {
                this->logger.LogDebug() << "image encoded to H.265 successfully";
            }
            else
            {
                this->logger.LogError() << "image H.265 encoding error with result: 0x" << failed;
                return failed;
            }
        }
        else
        {
            this->logger.LogError() << "image converting to BGR error with result: 0x" << failed;
            return failed;
        }
    }
    else
    {
        this->logger.LogError() << "image cropping and resizing error with result: 0x" << failed;
        return failed;
    }
    return failed;
}